#!/bin/ksh
#
# alert.sh: ver 2.0
# Lance Spitzner lance@spitzner.net
#
# Enhanced by:
# Aaron Turner aturner@pobox.com aka aturner@vicinity.com
#
# Parse, log, and react to FW-1 User Defined Alerts
# For more information, please review the README
#


#################################################################
# 		BEGIN CUSTOMIZING SCRIPT HERE			#
#################################################################

# Mail command
mail=/usr/bin/mailx

# INSTALL DIRECTORY
# Define the directory that this script is in.
# Do NOT put a slash at the end.
# EXAMPLE: dir=/home/fwadmin/alert
dir=

# FW ADMIN
# Define the name of who gets the email alerts
# EXAMPLE: user=fwadmin@example.com
user=

# SCAN LIMIT
# Define maixmum number of scans/email alerts
limit=5

# EMAIL REMOTE SYSTEM
# Define as "true" if you want to automatically email
# the remote admin when you reach your scan limit.
email=false

# SAM
# Define as "true" if you want to autotmatically block
# the source if you reach your scan limit.
sam=false

# SAM TIMEOUT
# How long do you want the source blocked
# Default is 3600 seconds (1 hour).
timeout=3600

# SAM_SERVICE_BLOCK
# Define as "true" if you want to automatically block
# the source for ONLY the offending service. 
# DO NOT enable this and sam (above) at the same time.
sam_service_block=false

# SAM_SERVICES
# Did you enable SAM per-service blocking?  If so, list those 
# services to protect here, space delimited.  
# EXAMPLE: sam_services="pop imap telnet ftp"
sam_services=""

# SAM_PROTECT_LIST
# This is a space seperated list of servers (hostname or dotted-notation)
# Which you want to protect with the $sam_service's
# Note: you can not use Object Names, but you can use DNS/IP.
# EXAMPLE: sam_protect_list="1.2.3.4 mailserver ftpserver"
sam_protect_list=""

# DONT_WAIT_FOR_LIMIT
# Do you want to enable SAM Service blocking without waiting for
# them to reach $limit scans?  yes/no
# This will generate a SAM/email for *each* scanned port/server if yes
# but only for services listed in $sam_service
dont_wait_for_limit=yes

# SAM_OPTIONS
# This is where you should put any options you would like to give to 
# the 'fw sam' command.  For more info run 'fw sam'
# Valid Options are: -v, -f, and -s
# EXAMPLE: sam_options="-v"
# WARNING: DO NOT USE -f if you are using sam_dynamic_enforce below.
sam_options="-v"

# SAM_INHIBIT
# Set this to either -i, -I, or -n depending on how you want SAM to
# inhibit connections.  For more info run 'fw sam'
sam_inhibit="-i"

# SAM_SERVICE_IGNORE_LIMIT
# Set to true if you want SAM_SERVICE blocking to be continued
# after $limit has been reached.  If true, you will not be notified via
# email any further, though you can scan the firwall logs for service 18183
# to see what happened.
sam_service_ignore_limit=false

# ENABLE_SAM_DYNAMIC_ENFORCE
# Set this to true, if you want to enable logic which allows you to 
# define dynamically which firewall to send the SAM to.  WARNING!
# You should be very careful about this!  It is HIGHLY recommended
# that you do not enable this until you have been using this script for
# a while.
# Note that this will effect both SAM block and SAM Service block.
# Use of this feature is recommended for large enterprise environments with many firewalls.
enable_sam_dynamic_enforce=false

# SAM_DYNAMIC_ENFORCE
# This is the sed command the script uses to determine which firewall(s)
# to do the enforment on.  Format is:
# /<firewall-name>/s/<firewall-name>/<firewall-object>/
# where <firewall-name> is the regexp of the firewall in the alert log
# and <firewall-object> is a Object Name of a firewall or group of firewalls.
# Note that the <firewall-name> may not be what you think it is.  You'll have to read 
# the alert log to find out.  (it's the 4th column).
# A -f in the $sam_options variable (above) will override this option
# Remember, this is case senstive!
# Examples:
# /FW1-A-EXT-NAT/s/FW1-A-EXT-NAT/FW1-A/
# /FW1-B-EXT-NAT/s/FW1-B-EXT-NAT/FW1-B/
# /East-[-A-Z]*/s/East-[-A-Z]*/East-Firewalls/
# /West-[-A-Z]*/s/West-[-A-Z]*/West-Firewalls/
cat <<EOF >/tmp/.sam_enforce_$$
# put your rules below, in '/match/s/match/change/' format

# Place no more rules blow this line
# if we have a match, stop processing, else use default rule
t
# Default rule, if no match above.  
# If you don't want SAM to be enforced on all firewalls, change the line below
# run 'fw sam' for more info
/.*/s/.*/All/
EOF


		 

#################################################################
# 		FINISH CUSTOMIZING SCRIPT HERE			#
#################################################################



#################################################################
#                   DEFINE SYSTEM VARIABLES HERE	        #
#################################################################


### Script variables
message=/tmp/.message_$$
send=/tmp/.send_$$
sam_output=/tmp/.sam_output_$$

### Good code is secure code
umask=177
PATH=/usr/bin:/sbin:/usr/sbin:/usr/local/bin
export PATH

if test -a $message
then
	rm $message
fi

if test -a $send
then
	rm $send
fi
if test -a $sam_output
then
	rm $sam_output
fi

### Set trap in case of abrupt exit
trap "rm $send $message ; exit 5" 1 2 15

### Grab User Defined Alert log, pipe to $message.
cat - | tail -1 > $message

### Parse log file
date=`awk '{print $1}' $message`
time=`awk '{print $2}' $message`
firewall_log=`awk '{print $4}' $message`
proto=`awk '{print $8}' $message`
ip=`awk '{print $10}' $message`
dst=`awk '{print $12}' $message`
service=`awk '{print $14}' $message`

# Archive $ip in $ipaddr for later use.
ipaddr=$ip

### Determine number of scans.
number=`grep -c $ip $dir/alert.log`
scan=`expr $number + 1`

### Check number of scans.  If we have reached our limit, lets bail
### now and save CPU cycles, unless $sam_service_ignore_limit is true
if [ $scan -gt $limit ]  && [ $sam_service_ignore_limit != true ];then
	cat $message >> $dir/alert.log
        rm $message 
        exit 10  
fi

# Set SAM Output splitter (this will print out if $sam_options contains -v)
cat <<EOF >$sam_output

---- BEGIN SAM OUTPUT ----

EOF

# See if they're using the -f option in $sam_options
USING_F="`echo $sam_options | grep -c "\-f"`"

# Determine if we enable dynamic enforcement, if so, determine the 
# enforcement points to use.
if [ $enable_sam_dynamic_enforce = true ] && [ $USING_F = 0 ];then
    firewall="`echo $firewall_log | sed -f /tmp/.sam_enforce_$$`"
    enforce="-f `echo $firewall`"
    rm /tmp/.sam_enforce_$$
else
    # This is only used for cosmetic/informational purposes
    firewall="All"
fi

#################################################################
# 			    FUNCTIONS   			#
#								#
#   Build your own modules here, such as paging or snmp_trap	#
#   alerts, then add them to Phase 3 in the script.		#
#################################################################


## This function checks to see if the source is an IP or hostname.
## If the source is an IP, it is resolved to a domain name.

Resolve () {

	echo $ip | awk -F"." '{print $NF }' | grep [0-9] >/dev/null

	if [ "$?" -eq 0 ]; then
		tmp=`nslookup $ip | grep Name: | awk '{print $2 }'`

		if [ "$tmp" != "" ]; then
			src=$tmp
		else
			src=$ip
		fi
	else
		src=$ip
	fi
	 

}

## This function will try to do a reverse lookup on $ipaddr if it 
## isn't an IP.

Reverse () {

    echo $ipaddr | awk -F"." '{print $NF }' | grep [a-zA-Z] >/dev/null

    if [ "$?" -eq 0 ];then
	tmp=`nslookup $ipaddr | grep Address: | awk '{print $2 }'`
	tmp2=`echo $tmp | awk -F" " '{print $2 }'`

	if [ "$tmp2" != "" ]; then
	    ipaddr=$tmp2
	fi
    fi
	 

}
	    

### This function determines who the admin is of the remote system
### and emails them about the scan.  Works only for .com, .edu, .net
### .mil, and .org.

TrackDown () {

# Define variables
dom=`echo $src | awk -F"." '{print $NF }'`
full_dom=`echo $src | awk -F"." '{print $(NF-1) }'`.$dom

# Start script, 1st check if it is a gTLD, if so, execute the following.
	if [ $dom = net ] || [ $dom = com ] || [ $dom = edu ] || [ $dom = org ] || [ $dom = mil ] || [ $dom = gov ]; then
		admin=`whois $full_dom | grep "@" | awk '{print $NF}' | head -1`
		tech=`whois $full_dom | grep "@" | awk '{print $NF}' | head -2 | tail -1`


mail $admin,$tech,$user <<EOF
Subject: Your system $src may be scanning the Internet.

I logged your system $src scanning my network.
It looks like they are scanning for the $service vulnerability.
I recommend you research this, as $src
may be scanning other networks as well.  Please respond to this
message informing me what you have found.  If you require
assistance from me, I would be more then happy to provide it.  
Below is an example one of the multiple connections I received
from $src.

--- Logged Information ---

IP:       $ipaddr
Source:   $src
Date:     $date
Time:     $time
Service:  $service

--- snip snip ---

This notification has been sent to 
$admin
$tech 
abuse@$full_dom

Please respond to $user for any issues concerning this.  
If you received this message in error, I apologize.

Thank you
EOF

	fi

}

## This function blocks the source IP scanning/probing our 
## network.  


Block () {
	$FWDIR/bin/fw sam -t $timeout $sam_options $enforce $sam_inhibit src $src >>$sam_output 2>&1

cat <<EOF >> $send
	
**** WARNING ****
Intruder $src ($ipaddr) has been temporarily blocked at the firewall ($firewall).  $src will be blocked for the next $timeout seconds.  

To re-enable $src, type the following command on the SAM server:

$FWDIR/bin/fw sam -t $timeout -C $sam_options $enforce $sam_inhibit src $ip
EOF
subject="#### CLIENT BLOCK ${src}:${service} ####"
}

## Service Block.
## This function blocks clients to specific hosts and ports rather than globally like Block does.

ServBlk () {
    for i in $sam_services; do
	if [ $i = $service ]; then
	    for sam_protect in $sam_protect_list; do
		$FWDIR/bin/fw sam $sam_options $enforce -t $timeout $sam_inhibit srv $src $sam_protect $service $proto >>$sam_output 2>&1
		echo "\n" >>$sam_output
		subject="#### SERVICE BLOCK ${src}:${service} ####"
		sam_protect_list_match=true
	    done
	fi
    done


# Don't put this text in the message unless we actually matched a $sam_protect_list service
if [ $sam_protect_list_match = true ]; then

cat <<EOF >> $send

**** WARNING ****
Intruder $src ($ipaddr) has been temporarily blocked at the firewall ($firewall) for the $service service.  $src will be blocked for the next $timeout seconds.

To re-enable $src for $service, type the following command(s) on the Firewall:

EOF

for i in $sam_services; do
    if [ $i = $service ]; then
	for sam_protect in $sam_protect_list; do
	    echo "$FWDIR/bin/fw sam $sam_options $enforce -C $sam_inhibit srv $src $sam_protect $service $proto" >>$send
	done
    fi
done

fi

}


#################################################################
# 			   THE SCRIPT   			#
#################################################################


### Resolve the IP address of $src if it is still in IP format.
### You may want to disable this to improve performance.
Resolve

### Sometimes Firewall-1 does a DNS lookup and gives us a name
### instead of an IP, and we want an IP.
### You may want to disable this to improve performance.
Reverse

### Set the default subject line of the email
### We change this in ServBlk/Block if applicable
subject="#### SCAN ALERT ${src}:${service} ####"

### Build Email Alert
### This builds our email alert.  This happens for
### every alert, until we exceed our $limit.  If we exceed
### our limit, the script never gets to this point, because
### it will have already exited (see around line 95).

cat <<EOF > $send

You have received this message because someone is potentially 
scanning your systems.  The information below is the packet 
that was denied and logged by the Firewall. This is email alert
number $scan, with a limit of $limit from $src. 

	----- CRITICAL INFORMATION -----

	Date:        $date
	Time:        $time
	Source:      $src
        IP:          $ipaddr
	Destination: $dst
	Service:     $service

	----- ACTUAL FW-1 LOG ENTRY -----

`cat $message` 
EOF




### PHASES START: 
### What to do in addition to email alerts, depending on
### the number of scans.

# ##### PHASE 1 #####
# First unauthorized connection from the remote system.

if [ $scan -eq 1 ]; then

	## Add this system to our scan database file alert.uniq
	echo "$src	$date	$time 	$service" >> $dir/alert.uniq

	if [ $dont_wait_for_limit = yes ] && [ $sam_service_block = true ]; then
	    ServBlk
	fi

# ##### PHASE 2 #####
# Second to $limit connections from the remote system

elif [ $scan -gt 1 ] && [ $scan -lt $limit ]; then

	## Check to see if we should do a service block
	if [ $dont_wait_for_limit = yes ] && [ $sam_service_block = true ]; then
	    ServBlk
	fi

# ##### PHASE 3 #####
# We are pretty sure this is a port scan or probe, since the
# same source has connected to us $limit number of times.

else

	### Track down and email the admin of the remote system
	if [ $email = true ]; then
		TrackDown
	fi

	### Block source 
	if [ $sam = true ]  && [ ! $scan -gt $limit ]; then
		Block
	### service block
	elif [ $sam_service_block = true ]; then
		ServBlk
	fi

	echo "\nThis is alert number $scan, you have reached your" >> $send
	echo "maximum threshold. You will not receive anymore alerts". >> $send	

fi


### Send email alert and save log to alert.log

if [ ! $scan -gt $limit ];then
    if [ `cat $sam_output | wc -l` -gt 3 ]; then # we got output from the sam commands...
	cat $sam_output >>$send
    fi
    $mail -s "$subject" $user < $send
fi

cat $message >> $dir/alert.log

## All done, lets clean up after ourselves, just like Mom taught us :)
rm $message
rm $send
rm $sam_output

exit 0

